local _, addon = ...
local QuestDemos = addon.QuestDemos
local CreateFrame = addon.G.CreateFrame

local menu = addon.MainMenu:NewMenuScreen("demo")

local colinfo = {
  {
    name = "Quest",
    pwidth = 0.7,
    align = "LEFT"
  },
  {
    name = "Faction",
    pwidth = 0.3,
    align = "RIGHT"
  }
}

local dqRows = {}
local buttons = {}

local function getDemoQuests()
  return dqRows
end

local function setButtonState(row)
  if row then
    buttons[1]:Enable()
    buttons[2]:Enable()
    buttons[3]:Enable()
  else
    buttons[1]:Disable()
    buttons[2]:Disable()
    buttons[3]:Disable()
  end
end

function menu:Create(frame)
  for _, dq in pairs(QuestDemos:FindAll()) do
    table.insert(dqRows, { dq.demoName, dq.faction, dq.order, dq.demoId })
  end
  table.sort(dqRows, function(a, b) return a[3] < b[3] end)

  local buttonPane = addon.CustomWidgets:CreateWidget("ButtonPane", frame, "LEFT", 120)

  local tablePane = CreateFrame("Frame", nil, frame)
  tablePane:SetPoint("TOPLEFT", buttonPane, "TOPRIGHT")
  tablePane:SetPoint("BOTTOMLEFT", buttonPane, "BOTTOMRIGHT")
  tablePane:SetPoint("TOPRIGHT", frame, "TOPRIGHT")
  tablePane:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT")

  local dataTable = addon.CustomWidgets:CreateWidget("DataTable", tablePane, colinfo, getDemoQuests)
  dataTable:RefreshData()
  dataTable:OnRowSelected(setButtonState)
  frame.dataTable = dataTable

  local acceptQuest = function()
    local row = dataTable:GetSelectedRow()
    if not row then return end
    addon.QuestDemos:StartDemo(row[4])
    dataTable:ClearSelection()
  end

  local viewCode = function()
    local selectedRow = dataTable:GetSelectedRow()
    if not selectedRow then return end
    local demoId = selectedRow[4]
    addon.MainMenu:ShowMenuScreen("demo-view", demoId)
  end

  local copyToDrafts = function()
    local row = dataTable:GetSelectedRow()
    if not row or not row[4] then
      return
    end
    local demo = addon.QuestDemos:FindByID(row[4])
    addon.StaticPopups:Show("RenameDemoCopy", demo)
  end

  buttons[1] = buttonPane:AddButton("Start Quest", acceptQuest)
  buttons[2] = buttonPane:AddButton("View Code", viewCode)
  buttons[3] = buttonPane:AddButton("Copy to Drafts", copyToDrafts)

  setButtonState(nil)
end

function menu:OnShow(frame)
  setButtonState(frame.dataTable:GetSelectedRow())
end